//
// Created by philipp on 21.01.18.
//

#include "../include/Busyable.h"

namespace networking
{

bool Busyable::isBusy() const noexcept
{
    return busy;
}

BusyLock::BusyLock(Busyable & busyable) : busyable(&busyable)
{
    if (!busyable.busyMutex.try_lock())
        throw error::Busy{};

    owns = true;
    busyable.busy = true;
}

BusyLock::~BusyLock()
{
    if (owns)
        unlock();
}

BusyLock::BusyLock(BusyLock && other) noexcept
    : busyable(other.busyable)
      , owns(other.owns.load())
{
    other.busyable = nullptr;
    other.owns = false;
}

BusyLock & BusyLock::operator=(BusyLock && other) noexcept
{
    if (owns)
        unlock();

    busyable = other.busyable;
    owns = other.owns.load();

    other.busyable = nullptr;
    other.owns = false;
}

void BusyLock::unlock()
{
    busyable->busy = false;
    busyable->busyMutex.unlock();
    owns = false;
}

}