//
// Created by philipp on 05.02.18.
//

#ifndef PC2CAR_COMMANDRECEIVER_H
#define PC2CAR_COMMANDRECEIVER_H

#include "PC2Car.h"
#include <NetworkingLib/DatagramReceiver.h>
#include <PlatoonProtocolLib/Protocol.h>
#include "TimedValue.h"
#include "json.hpp"

namespace pc2car
{

class CommandReceiver : public std::enable_shared_from_this<CommandReceiver>
{
private:
    struct PrivateTag
    {
    };

public:
    using Ptr = std::shared_ptr<CommandReceiver>;
    using DatagramReceiver = networking::message::DatagramReceiver<std::string>;
    using OnCommandReceivedCallback = std::function<void(CommandCode)>;

    static Ptr create(
        networking::Networking & net,
        bool defaultLoggingEnabled = false,
        bool defaultPlatoonEnabled = false,
        bool defaultRCModeEnabled = false,
        float defaultPlatoonSpeed = 0.0f,
        float defaultInnerPlatoonDistance = 0.0f,
        float defaultSpeed = 0.0f);

    CommandReceiver(PrivateTag,
                    networking::Networking & net,
                    bool defaultLoggingEnabled,
                    bool defaultPlatoonEnabled,
                    bool defaultRCModeEnabled,
                    float defaultPlatoonSpeed,
                    float defaultInnerPlatoonDistance,
                    float defaultSpeed);

    void receiveCommands(const OnCommandReceivedCallback & handler);

    void stop();

    TimedValue<bool> isLoggingEnabled()
    { return loggingEnabled.getNonAtomicCopy(); }

    TimedValue<bool> isPlatoonEnabled()
    { return platoonEnabled.getNonAtomicCopy(); }

    TimedValue<bool> isRcModeEnabled()
    { return rcModeEnabled.getNonAtomicCopy(); }

    TimedValue<platoonProtocol::PlatoonSpeed> getPlatoonSpeed()
    { return platoonSpeed.getNonAtomicCopy(); }

    TimedValue<platoonProtocol::InnerPlatoonDistance> getInnerPlatoonDistance()
    { return innerPlatoonDistance.getNonAtomicCopy(); }

    TimedValue<float> getSpeed()
    { return speed.getNonAtomicCopy(); }

private:
    DatagramReceiver::Ptr receiver;
    OnCommandReceivedCallback handler;

    TimedAtomicValue<bool> loggingEnabled{false};
    TimedAtomicValue<bool> platoonEnabled{false};
    TimedAtomicValue<bool> rcModeEnabled{false};
    TimedAtomicValue<platoonProtocol::PlatoonSpeed> platoonSpeed{false};
    TimedAtomicValue<platoonProtocol::InnerPlatoonDistance> innerPlatoonDistance{false};
    TimedAtomicValue<float> speed{false};

    void receiveCommand();

    void setValue(CommandCode code, nlohmann::json & j);
};

}

#endif //PC2CAR_COMMANDRECEIVER_H
